/*! Tripetto Collector 1.7.1-54 - Copyright (C) 2020 Tripetto B.V. - All Rights Reserved */

declare module "@marblecore/markdown" {
    /** Explicit exports */
    export { IMarkdown, MarkdownParser } from "@marblecore/markdown/lib/parser";
    export { IMarkdownOptions } from "@marblecore/markdown/lib/options";
    export { MarkdownFeatures } from "@marblecore/markdown/lib/features";
    export { MarkdownTypes } from "@marblecore/markdown/lib/types";
}

declare module "@marblecore/markdown/lib/parser" {
    /** Dependencies */
    import { MarkdownTypes } from "@marblecore/markdown/lib/types";
    import { MarkdownFeatures } from "@marblecore/markdown/lib/features";
    import {
        IMarkdownOptions,
        IMarkdownRef
    } from "@marblecore/markdown/lib/options";
    export interface IMarkdown<T> {
        content: string | IMarkdownNode<T> | IMarkdownNode<T>[];
        type?: MarkdownTypes;
        value?: string | T | IMarkdownRef<T>;
    }
    export interface IMarkdownNode<T> extends IMarkdown<T> {
        parent?: IMarkdownNode<T>;
    }
    export class MarkdownParser<T = undefined> {
        /**
         * Parses the supplied markdown string and returns a markdown parser instance.
         * @param sMarkdown Specifies the markdown string to parse.
         * @param pOptions Specifies options for the markdown parser.
         * @return Returns a markdown parser instance.
         */
        static parse<T = undefined>(
            sMarkdown: string,
            pOptions?: IMarkdownOptions<T>
        ): MarkdownParser<T>;
        /**
         * Creates a new markdown parser instance.
         * @param sMarkdown Specifies the markdown string to parse.
         * @param pOptions Specifies options for the markdown parser.
         */
        constructor(sMarkdown: string, pOptions?: IMarkdownOptions<T>);
        /** Contains the enabled features. */
        readonly features: MarkdownFeatures;
        /** Retrieves a reference to the AST. */
        readonly abstractSyntaxTree: IMarkdown<T>;
        /** Retrieves the plain text without markdown formatting. */
        readonly text: string;
        /** Retrieves if a mention with a value is parsed. */
        readonly hasMentions: boolean;
        /** Retrieves if a reference with a value is parsed. */
        readonly hasReferences: boolean;
        /**
         * Maps the markdown AST to a structure.
         * @param fnMap Specifies the map function.
         * @return Returns the ancestor of the map structure.
         */
        map<R>(
            fnMap: (
                sType: MarkdownTypes | undefined,
                sContent: string,
                pValue: string | T | undefined,
                pParent: R | undefined,
                sKind: "root" | "text" | "node"
            ) => R
        ): R;
        /**
         * Reduces the markdown AST.
         * @param fnReduce Specifies the reduce function.
         * @returns Returns the reduced value.
         */
        reduce<R>(
            fnReduce: (
                sType: MarkdownTypes | undefined,
                pContent: string | R[],
                pValue: string | T | undefined
            ) => R
        ): R;
    }
}

declare module "@marblecore/markdown/lib/options" {
    import { MarkdownFeatures } from "@marblecore/markdown/lib/features";
    export interface IMarkdownRef<T> {
        ref: T;
        string: string;
    }
    export interface IMarkdownBufferParser<T> {
        /**
         * The parser function.
         * @param sBuffer Contains the buffer.
         * @return Should return an object with the following properties:
         * - `shift`: Specifies the number of characters to shift the input buffer;
         * - `markdown`: Specifies the markdown string that is pushed to the AST (optional);
         * - `text`: Specifies the text string that is pused to the AST when no
         * markdown string is supplied (optional);
         * - `value`: Specifies the value that is pushed to the AST (optional).
         * Or return `undefined` to skip the parser.
         */
        parser: (
            sBuffer: string
        ) =>
            | {
                  shift: number;
                  markdown?: string;
                  text?: string;
                  value?: T | IMarkdownRef<T>;
              }
            | undefined;
        /**
         * Requested number of chars from the buffer. If you omit this property,
         * the complete buffer is supplied to the parser function.
         */
        numberOfChars?: number;
    }
    export interface IMarkdownOptions<T> {
        /** Specifies the parser features. */
        features?: MarkdownFeatures;
        /** Specifies a `@`-mention parser. */
        mentions?: IMarkdownBufferParser<T>;
        /** Specifies a `#`-reference parser. */
        references?: IMarkdownBufferParser<T>;
        /** Specifies a stop word for the parser. Parsing stops when the word is found. */
        stopWord?: string;
    }
}

declare module "@marblecore/markdown/lib/features" {
    export enum MarkdownFeatures {
        /** Disable all features. */
        None = 0,
        /** Allow basic formatting like bold, italic, underline and strikethrough. */
        Formatting = 1,
        /** Allow the use of paragraphs (necessary for the `Headings`, `Quotes` or `Code` features). */
        Paragraphs = 2,
        /** Allow inline breaks. */
        Breaks = 4,
        /** Allow hyperlinks. */
        Hyperlinks = 8,
        /** Allow headings. */
        Headings = 16,
        /** Allow block quotes. */
        Blockquotes = 32,
        /** Allow code blocks. */
        Code = 64,
        /** Allows inline code. */
        InlineCode = 128,
        /** Allow embedding of images. */
        Images = 256,
        /** All features. */
        All = 511
    }
}

declare module "@marblecore/markdown/lib/types" {
    export type MarkdownTypes =
        | "paragraph"
        | "break"
        | "bold"
        | "italic"
        | "bold+italic"
        | "underline"
        | "strikethrough"
        | "code"
        | "hyperlink"
        | "image"
        | "h1"
        | "h2"
        | "h3"
        | "h4"
        | "h5"
        | "h6"
        | "codeblock"
        | "codeline"
        | "quote"
        | "mention"
        | "reference";
}
